# -*- coding: UTF-8 -*

"""Notes, décisions de jury, évènements scolaires
"""

from app import db

import app.scodoc.notesdb as ndb
import app.scodoc.sco_utils as scu


class BulAppreciations(db.Model):
    """Appréciations sur bulletins"""

    __tablename__ = "notes_appreciations"
    id = db.Column(db.Integer, primary_key=True)
    date = db.Column(db.DateTime(timezone=True), server_default=db.func.now())
    etudid = db.Column(
        db.Integer,
        db.ForeignKey("identite.id", ondelete="CASCADE"),
        index=True,
    )
    formsemestre_id = db.Column(
        db.Integer,
        db.ForeignKey("notes_formsemestre.id"),
    )
    author = db.Column(db.Text)  # le pseudo (user_name), sans contrainte
    comment = db.Column(db.Text)  # texte libre


class NotesNotes(db.Model):
    """Une note"""

    __tablename__ = "notes_notes"
    __table_args__ = (db.UniqueConstraint("etudid", "evaluation_id"),)
    id = db.Column(db.Integer, primary_key=True)
    etudid = db.Column(
        db.Integer,
        db.ForeignKey("identite.id", ondelete="CASCADE"),
    )
    evaluation_id = db.Column(
        db.Integer, db.ForeignKey("notes_evaluation.id"), index=True
    )
    value = db.Column(db.Float)
    # infos sur saisie de cette note:
    comment = db.Column(db.Text)  # texte libre
    date = db.Column(db.DateTime(timezone=True), server_default=db.func.now())
    uid = db.Column(db.Integer, db.ForeignKey("user.id"))

    def __init__(self, etudid, evaluation_id, value, comment, date, uid):
        self.etudid = etudid
        self.evaluation_id = evaluation_id
        self.value = value
        self.comment = comment
        self.date = date
        self.uid = uid

    def to_dict(self):
        return {
            "id": self.id,
            "etudid": self.etudid,
            "evaluation_id": self.evaluation_id,
            "value": self.value,
            "comment": self.comment,
            "date": self.date,
            "uid": self.uid,
        }


class NotesNotesLog(db.Model):
    """Historique des modifs sur notes (anciennes entrees de notes_notes)"""

    __tablename__ = "notes_notes_log"
    id = db.Column(db.Integer, primary_key=True)

    etudid = db.Column(
        db.Integer,
        db.ForeignKey("identite.id", ondelete="CASCADE"),
    )
    evaluation_id = db.Column(
        db.Integer,
        # db.ForeignKey("notes_evaluation.id"),
        index=True,
    )
    value = db.Column(db.Float)
    # infos sur saisie de cette note:
    comment = db.Column(db.Text)  # texte libre
    date = db.Column(db.DateTime(timezone=True), server_default=db.func.now())
    uid = db.Column(db.Integer, db.ForeignKey("user.id"))


def etud_has_notes_attente(etudid, formsemestre_id):
    """Vrai si cet etudiant a au moins une note en attente dans ce semestre.
    (ne compte que les notes en attente dans des évaluations avec coef. non nul).
    """
    cursor = db.session.execute(
        """SELECT COUNT(*)
        FROM notes_notes n, notes_evaluation e, notes_moduleimpl m,
        notes_moduleimpl_inscription i
        WHERE n.etudid = :etudid
        and n.value = :code_attente
        and n.evaluation_id = e.id
        and e.moduleimpl_id = m.id
        and m.formsemestre_id = :formsemestre_id
        and e.coefficient != 0
        and m.id = i.moduleimpl_id
        and i.etudid = :etudid
        """,
        {
            "formsemestre_id": formsemestre_id,
            "etudid": etudid,
            "code_attente": scu.NOTES_ATTENTE,
        },
    )
    return cursor.fetchone()[0] > 0
